{-------------------------------------------------------------------------------
Copyright (c) 2010, Ortus bvba, Belgium
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
* Neither the name of "Ortus" nor the names of its contributors may be used to
  endorse or promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-------------------------------------------------------------------------------}

{$ObjExportAll On}
unit OrtusShellBrowseComputer;

interface

uses
  Windows, Messages, Classes, SysUtils, StdCtrls, Controls, Graphics, Forms,
  ShellApi, ShlObj, ActiveX;

type
  TOrtusShellBrowseComputer= class(TComponent)
  private
    FComputerName: string;
    FCaption: string;
  protected
    procedure Loaded; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    function Execute: Boolean;

    property ComputerName: string read FComputerName;
  published
    property Caption: string read FCaption write FCaption;
  end;

implementation

procedure FreePIDL(PIDL: PItemIDList);
var
  ShellMalloc: IMalloc;
begin
  SHGetMalloc(ShellMalloc);
  try
    if PIDL <> nil then ShellMalloc.Free(PIDL);
  finally
    ShellMalloc := nil;
  end {try};
end;

//------------------------------------------------------------------------------
// TOrtusShellBrowseComputer
//------------------------------------------------------------------------------

procedure TOrtusShellBrowseComputer.Loaded;
begin
  inherited Loaded;
end;

constructor TOrtusShellBrowseComputer.Create(Owner: TComponent);
begin
  inherited Create(Owner);
end;

destructor TOrtusShellBrowseComputer.Destroy;
begin
  inherited Destroy;
end;

function TOrtusShellBrowseComputer.Execute: Boolean;
var
  BrowseInfo: TBrowseInfo;
  PIDL: PItemIDList;
  P: array[0..MAX_PATH] of Char;
  WindowList: Pointer;
begin
  Result := False;
  if SHGetSpecialFolderLocation(Application.Handle, CSIDL_NETWORK, PIDL) <> NOERROR then Exit;
  try
    FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
    BrowseInfo.hwndOwner := Application.Handle;
    BrowseInfo.pidlRoot := PIDL;
    BrowseInfo.pszDisplayName := P;
    BrowseInfo.lpszTitle := PChar(FCaption);
    BrowseInfo.ulFlags := BIF_BROWSEFORCOMPUTER;
    WindowList := DisableTaskWindows(0);
    try
      Result := SHBrowseForFolder(BrowseInfo) <> nil;
    finally
      EnableTaskWindows(WindowList);
    end;
    if Result then FComputerName := StrPas(P);
  finally
    FreePIDL(PIDL);
  end {try};
end;

end.
