{-------------------------------------------------------------------------------
Copyright (c) 2010, Ortus bvba, Belgium
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
* Neither the name of "Ortus" nor the names of its contributors may be used to
  endorse or promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-------------------------------------------------------------------------------}

{$ObjExportAll On}
unit OrtusShellBrowseFolder;

interface

uses
  Windows, Messages, Classes, SysUtils, StdCtrls, Controls, Graphics, Forms,
  ShellApi, ShlObj, ActiveX;

type
  TOrtusShellBrowseFolder= class(TComponent)
  private
    FFolderName: string;
    FCaption: string;
    FInitialFolderName: string;
  protected
    procedure Loaded; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    function Execute: Boolean;

    property FolderName: string read FFolderName;
  published
    property Caption: string read FCaption write FCaption;
    property InitialFolderName: string read FInitialFolderName write FInitialFolderName;
  end;

implementation

procedure FreePIDL(PIDL: PItemIDList);
var
  ShellMalloc: IMalloc;
begin
  SHGetMalloc(ShellMalloc);
  try
    if PIDL <> nil then ShellMalloc.Free(PIDL);
  finally
    ShellMalloc := nil;
  end {try};
end;

//------------------------------------------------------------------------------
// TOrtusShellBrowseFolder
//------------------------------------------------------------------------------

procedure TOrtusShellBrowseFolder.Loaded;
begin
  inherited Loaded;
end;

constructor TOrtusShellBrowseFolder.Create(Owner: TComponent);
begin
  inherited Create(Owner);
end;

destructor TOrtusShellBrowseFolder.Destroy;
begin
  inherited Destroy;
end;

function Browsecallback(Wnd: HWnd; Msg: UINT; Param: LPARAM; Data: LPARAM): Integer; stdcall;
begin
  if Msg = BFFM_INITIALIZED then
    SendMessage(Wnd, BFFM_SETSELECTION, 1, Integer(PChar(TOrtusShellBrowseFolder(Data).InitialFolderName)));
  Result := 0;
end;

function TOrtusShellBrowseFolder.Execute: Boolean;
var
  BrowseInfo: TBrowseInfo;
  RootPIDL: PItemIDList;
  FolderPIDL: PItemIDList;
  P: array[0..MAX_PATH] of Char;
  WindowList: Pointer;
begin
  RootPIDL := nil;
  FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
  BrowseInfo.hwndOwner := Application.Handle;
  BrowseInfo.pidlRoot := RootPIDL;
  BrowseInfo.pszDisplayName := P;
  BrowseInfo.lpszTitle := PChar(FCaption);
  BrowseInfo.lpfn := @BrowseCallback;
  BrowseInfo.lParam := Integer(Self);
  BrowseInfo.ulFlags := BIF_RETURNONLYFSDIRS;
  WindowList := DisableTaskWindows(0);
  try
    FolderPIDL := SHBrowseForFolder(BrowseInfo);
    Result := FolderPIDL <> nil;
    if not Result then
      FFolderName := ''
    else begin
      SHGetPathFromIDList(FolderPIDL, P);
      FFolderName := StrPas(P);
    end;
  finally
    EnableTaskWindows(WindowList);
  end {try};
end;

end.
