{-------------------------------------------------------------------------------
Copyright (c) 2010, Ortus bvba, Belgium
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
* Neither the name of "Ortus" nor the names of its contributors may be used to
  endorse or promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-------------------------------------------------------------------------------}

{$ObjExportAll On}
unit OrtusShellControlPanel;

interface

uses
  Windows, Messages, Classes, SysUtils, StdCtrls, Controls, Graphics, Forms,
  ShellApi;

type
  TOrtusShellCPL= (
    cplControlPanel,
    cplAccessibilityOptions,
    cplAddRemovePrograms,
    cplDisplayOptions,
    cplRegionalSettings,
    cplJoystickOptions,
    cplMouseProperties,
    cplKeyboardProperties,
    cplSoundAndMultimediaProperties,
    cplModemOptions,
    cplSystemProperties,
    cplDateTimeOptions,
    cplAddHardware,
    cplInternetProperties,
    cplAddPrinter,
    cplViewPrinters,
    cplViewFonts
  );

  TOrtusShellControlPanel= class(TComponent)
  private
    FCPL: TOrtusShellCPL;
  protected
    procedure Loaded; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    function Execute: Boolean;
  published
    property CPL: TOrtusShellCPL read FCPL write FCPL default cplControlPanel;
  end;

implementation

//------------------------------------------------------------------------------
// TOrtusShellControlPanel
//------------------------------------------------------------------------------

procedure TOrtusShellControlPanel.Loaded;
begin
  inherited Loaded;
end;

constructor TOrtusShellControlPanel.Create(Owner: TComponent);
begin
  inherited Create(Owner);

  FCPL := cplControlPanel;
end;

destructor TOrtusShellControlPanel.Destroy;
begin
  inherited Destroy;
end;

function TOrtusShellControlPanel.Execute: Boolean;
var
  S: string;
begin
  case FCPL of
    cplControlPanel: S := 'shell32.dll,Control_RunDLL';
    cplAccessibilityOptions: S := 'shell32.dll,Control_RunDLL access.cpl';
    cplAddRemovePrograms: S := 'shell32.dll,Control_RunDLL appwiz.cpl';
    cplDisplayOptions: S := 'shell32.dll,Control_RunDLL desk.cpl';
    cplRegionalSettings: S := 'shell32.dll,Control_RunDLL intl.cpl';
    cplJoystickOptions: S := 'shell32.dll,Control_RunDLL joy.cpl';
    cplMouseProperties: S := 'shell32.dll,Control_RunDLL main.cpl @0';
    cplKeyboardProperties: S := 'shell32.dll,Control_RunDLL main.cpl @1';
    cplSoundAndMultimediaProperties: S := 'shell32.dll,Control_RunDLL mmsys.cpl';
    cplModemOptions: S := 'shell32.dll,Control_RunDLL modem.cpl';
    cplSystemProperties: S := 'shell32.dll,Control_RunDLL sysdm.cpl';
    cplDateTimeOptions: S := 'shell32.dll,Control_RunDLL timedate.cpl';
    cplAddHardware: S := 'shell32.dll,Control_RunDLL hdwwiz.cpl';
    cplInternetProperties: S := 'shell32.dll,Control_RunDLL inetcpl.cpl';
    cplAddPrinter: S := 'shell32.dll,SHHelpShortcuts_RunDLL AddPrinter';
    cplViewPrinters: S := 'shell32.dll,SHHelpShortcuts_RunDLL PrintersFolder';
    cplViewFonts: S := 'shell32.dll,SHHelpShortcuts_RunDLL FontsFolder';
  end {case};
  Result := ShellExecute(Application.Handle, nil, 'rundll32.exe', PChar(S), nil, SW_NORMAL) > 32;
end;

end.
