{-------------------------------------------------------------------------------
Copyright (c) 2010, Ortus bvba, Belgium
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
* Neither the name of "Ortus" nor the names of its contributors may be used to
  endorse or promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-------------------------------------------------------------------------------}

{$ObjExportAll On}
unit OrtusShellFormatDisk;

interface

uses
  Windows, Messages, Classes, SysUtils, StdCtrls, Controls, Graphics, Forms,
  ShellApi;

type
  TOrtusShellFormatDisk= class(TComponent)
  private
    FDrive: Char;

    procedure SetDrive(Value: Char);
  protected
    procedure Loaded; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    function Execute: Boolean;
  published
    property Drive: Char read FDrive write SetDrive;
  end;

implementation

//------------------------------------------------------------------------------
// TOrtusShellFormatDisk
//------------------------------------------------------------------------------

procedure TOrtusShellFormatDisk.SetDrive(Value: Char);
begin
  if not (UpCase(Value) in ['A'..'Z']) then Exit;
  FDrive := UpCase(Value);
end;

procedure TOrtusShellFormatDisk.Loaded;
begin
  inherited Loaded;
end;

constructor TOrtusShellFormatDisk.Create(Owner: TComponent);
begin
  inherited Create(Owner);

  FDrive := 'A';
end;

destructor TOrtusShellFormatDisk.Destroy;
begin
  inherited Destroy;
end;

function TOrtusShellFormatDisk.Execute: Boolean;
const
  SHFMT_ID_DEFAULT= $FFFF;
type
  TSHFormatDiskProc= function(Wnd: THandle; Drive: Integer; FmtId: Integer; Options: Integer): Cardinal; stdcall;
var
  ModuleHandle: THandle;
  FormatDriveProc: TSHFormatDiskProc;
begin
  Result := False;
  ModuleHandle := LoadLibrary('SHELL32.DLL');
  if ModuleHandle = 0 then Exit;
  try
    FormatDriveProc := GetProcAddress(ModuleHandle, PChar('SHFormatDrive'));
    if @FormatDriveProc = nil then Exit;
    FormatDriveProc(Application.Handle, Ord(FDrive) - Ord('A'), SHFMT_ID_DEFAULT, 0);
    Result := True;
  finally
    FreeLibrary(ModuleHandle);
  end {try};
end;

end.
