{-------------------------------------------------------------------------------
Copyright (c) 2010, Ortus bvba, Belgium
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
* Neither the name of "Ortus" nor the names of its contributors may be used to
  endorse or promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-------------------------------------------------------------------------------}

{$ObjExportAll On}
unit OrtusShellProperties;

interface

uses
  Windows, Messages, Classes, SysUtils, StdCtrls, Controls, Graphics, Forms,
  ShellApi;

type
  TOrtusShellProperties= class(TComponent)
  private
    FFileName: string;
  protected
    procedure Loaded; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    function Execute: Boolean;
  published
    property FileName: string read FFileName write FFileName;
  end;

implementation

//------------------------------------------------------------------------------
// TOrtusShellProperties
//------------------------------------------------------------------------------

procedure TOrtusShellProperties.Loaded;
begin
  inherited Loaded;
end;

constructor TOrtusShellProperties.Create(Owner: TComponent);
begin
  inherited Create(Owner);
end;

destructor TOrtusShellProperties.Destroy;
begin
  inherited Destroy;
end;

function TOrtusShellProperties.Execute: Boolean;
type
  TSHPropertiesProc= function(Wnd: THandle; Flags: Integer; ItemName: PChar; TabName: PChar): LongBool; stdcall;
var
  ModuleHandle: THandle;
  PropertiesProc: TSHPropertiesProc;
  FileNameBuf: Pointer;
begin
  Result := False;
  ModuleHandle := LoadLibrary('SHELL32.DLL');
  if ModuleHandle = 0 then Exit;
  try
    PropertiesProc := GetProcAddress(ModuleHandle, PChar(178));
    if @PropertiesProc = nil then Exit;
    if FFileName = '' then FileNameBuf := nil
    else begin
      // We have to use ANSI for Win95 and Win98, and UNICODE for WinNT or later
      GetMem(FileNameBuf, (Length(FFileName) + 1) * SizeOf(WideChar));
      if Win32Platform = VER_PLATFORM_WIN32_NT then
        StringToWideChar(FFileName, PWideChar(FileNameBuf), Length(FFileName) + 1)
      else
        StrPCopy(PChar(FileNameBuf), FFileName);
    end;
    Result := PropertiesProc(Application.Handle, $02, FileNameBuf, nil);
    if FileNameBuf <> nil then FreeMem(FileNameBuf, (Length(FFileName) + 1) * SizeOf(WideChar));
  finally
    FreeLibrary(ModuleHandle);
  end {try};
end;

end.
