{-------------------------------------------------------------------------------
Copyright (c) 2010, Ortus bvba, Belgium
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
* Neither the name of "Ortus" nor the names of its contributors may be used to
  endorse or promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-------------------------------------------------------------------------------}

{$ObjExportAll On}
unit OrtusShellRun;

interface

uses
  Windows, Messages, Classes, SysUtils, StdCtrls, Controls, Graphics, Forms,
  ShellApi;

type
  TOrtusShellRun= class(TComponent)
  private
    FCaption: string;
    FText: string;
    FIcon: TIcon;
  protected
    procedure Loaded; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    function Execute: Boolean;
  published
    property Caption: string read FCaption write FCaption;
    property Text: string read FText write FText;
    property Icon: TIcon read fIcon write FIcon;
  end;

implementation

//------------------------------------------------------------------------------
// TOrtusShellRun
//------------------------------------------------------------------------------

procedure TOrtusShellRun.Loaded;
begin
  inherited Loaded;
end;

constructor TOrtusShellRun.Create(Owner: TComponent);
begin
  inherited Create(Owner);

  FIcon := TIcon.Create;
end;

destructor TOrtusShellRun.Destroy;
begin
  FIcon.Free;

  inherited Destroy;
end;

function TOrtusShellRun.Execute: Boolean;
type
  TSHRunFileProc= function(Wnd: THandle; Icon: Integer; Directory: PChar; Title: PChar; Description: PChar; Flags: Integer): DWORD; stdcall;
var
  ModuleHandle: THandle;
  RunFileProc: TSHRunFileProc;
  CaptionBuf: Pointer;
  TextBuf: Pointer;
begin
  Result := False;
  ModuleHandle := LoadLibrary('SHELL32.DLL');
  if ModuleHandle = 0 then Exit;
  try
    RunFileProc := GetProcAddress(ModuleHandle, PChar(61));
    if @RunFileProc = nil then Exit;
    if FCaption = '' then CaptionBuf := nil
    else begin
      // We have to use ANSI for Win95 and Win98, and UNICODE for WinNT or later
      GetMem(CaptionBuf, (Length(FCaption) + 1) * SizeOf(WideChar));
      if Win32Platform = VER_PLATFORM_WIN32_NT then
        StringToWideChar(FCaption, PWideChar(CaptionBuf), Length(FCaption) + 1)
      else
        StrPCopy(PChar(CaptionBuf), FCaption);
    end;
    if FText = '' then TextBuf := nil
    else begin
      GetMem(TextBuf, (Length(FText) + 1) * SizeOf(WideChar));
      if Win32Platform = VER_PLATFORM_WIN32_NT then
        StringToWideChar(FText, PWideChar(TextBuf), Length(FText) + 1)
      else
        StrPCopy(PChar(TextBuf), FText);
    end;
    Result := RunFileProc(Application.Handle, FIcon.Handle, nil, CaptionBuf, TextBuf, 0) = 0;
    if CaptionBuf <> nil then FreeMem(CaptionBuf, (Length(FCaption) + 1) * SizeOf(WideChar));
    if TextBuf <> nil then FreeMem(TextBuf, (Length(FText) + 1) * SizeOf(WideChar));
  finally
    FreeLibrary(ModuleHandle);
  end {try};
end;

end.
